@@ -16,7 +16,6 @@
 
 """Command-line interface to the OpenStack APIs"""
 
-import argparse
 import getpass
 import locale
 import logging
@@ -39,11 +38,13 @@ from osc_lib import exceptions as exc
 from osc_lib.i18n import _
 from osc_lib import logs
 from osc_lib import utils
+from osc_lib import version
 
 osprofiler_profiler = importutils.try_import("osprofiler.profiler")
 
 
 DEFAULT_DOMAIN = 'default'
+DEFAULT_INTERFACE = 'public'
 
 
 def prompt_for_password(prompt=None):
@@ -148,30 +149,25 @@ class OpenStackShell(app.App):
             self.log.info("END return value: %s", ret_val)
 
     def init_profile(self):
-        # NOTE(dtroyer): Remove this 'if' block when the --profile global
-        #                option is removed
-        if osprofiler_profiler and self.options.old_profile:
-            self.log.warning(
-                'The --profile option is deprecated, '
-                'please use --os-profile instead'
-            )
-            if not self.options.profile:
-                self.options.profile = self.options.old_profile
-
         self.do_profile = osprofiler_profiler and self.options.profile
         if self.do_profile:
             osprofiler_profiler.init(self.options.profile)
 
     def close_profile(self):
         if self.do_profile:
-            trace_id = osprofiler_profiler.get().get_base_id()
+            profiler = osprofiler_profiler.get()
+            trace_id = profiler.get_base_id()
+            # Short ID for OpenTracing-based driver (64-bit id)
+            short_id = profiler.get_shorten_id(trace_id)
 
             # NOTE(dbelova): let's use warning log level to see these messages
             # printed. In fact we can define custom log level here with value
             # bigger than most big default one (CRITICAL) or something like
             # that (PROFILE = 60 for instance), but not sure we need it here.
             self.log.warning("Trace ID: %s" % trace_id)
-            self.log.warning("Display trace with command:\n"
+            self.log.warning("Short trace ID "
+                             "for OpenTracing-based drivers: %s" % short_id)
+            self.log.warning("Display trace data with command:\n"
                              "osprofiler trace show --html %s " % trace_id)
 
     def run_subcommand(self, argv):
@@ -260,10 +256,57 @@ class OpenStackShell(app.App):
             metavar='<interface>',
             dest='interface',
             choices=['admin', 'public', 'internal'],
-            default=utils.env('OS_INTERFACE'),
+            default=utils.env(
+                'OS_INTERFACE',
+                default=DEFAULT_INTERFACE),
             help=_('Select an interface type.'
                    ' Valid interface types: [admin, public, internal].'
-                   ' (Env: OS_INTERFACE)'),
+                   ' default=%s, (Env: OS_INTERFACE)') % DEFAULT_INTERFACE,
+        )
+        parser.add_argument(
+            '--os-service-provider',
+            metavar='<service_provider>',
+            dest='service_provider',
+            default=utils.env('OS_SERVICE_PROVIDER'),
+            help=_('Authenticate with and perform the command on a service'
+                   ' provider using Keystone-to-keystone federation. Must'
+                   ' also specify the remote project option.')
+        )
+        remote_project_group = parser.add_mutually_exclusive_group()
+        remote_project_group.add_argument(
+            '--os-remote-project-name',
+            metavar='<remote_project_name>',
+            dest='remote_project_name',
+            default=utils.env('OS_REMOTE_PROJECT_NAME'),
+            help=_('Project name when authenticating to a service provider'
+                   ' if using Keystone-to-Keystone federation.')
+        )
+        remote_project_group.add_argument(
+            '--os-remote-project-id',
+            metavar='<remote_project_id>',
+            dest='remote_project_id',
+            default=utils.env('OS_REMOTE_PROJECT_ID'),
+            help=_('Project ID when authenticating to a service provider'
+                   ' if using Keystone-to-Keystone federation.')
+        )
+        remote_project_domain_group = parser.add_mutually_exclusive_group()
+        remote_project_domain_group.add_argument(
+            '--os-remote-project-domain-name',
+            metavar='<remote_project_domain_name>',
+            dest='remote_project_domain_name',
+            default=utils.env('OS_REMOTE_PROJECT_DOMAIN_NAME'),
+            help=_('Domain name of the project when authenticating to a'
+                   ' service provider if using Keystone-to-Keystone'
+                   ' federation.')
+        )
+        remote_project_domain_group.add_argument(
+            '--os-remote-project-domain-id',
+            metavar='<remote_project_domain_id>',
+            dest='remote_project_domain_id',
+            default=utils.env('OS_REMOTE_PROJECT_DOMAIN_ID'),
+            help=_('Domain ID of the project when authenticating to a'
+                   ' service provider if using Keystone-to-Keystone'
+                   ' federation.')
         )
         parser.add_argument(
             '--timing',
@@ -283,18 +326,9 @@ class OpenStackShell(app.App):
                 '--os-profile',
                 metavar='hmac-key',
                 dest='profile',
+                default=utils.env('OS_PROFILE'),
                 help=_('HMAC key for encrypting profiling context data'),
             )
-            # NOTE(dtroyer): This global option should have been named
-            #                --os-profile as --profile interferes with at
-            #                least one existing command option.  Deprecate
-            #                --profile and remove after Apr 2017.
-            parser.add_argument(
-                '--profile',
-                metavar='hmac-key',
-                dest='old_profile',
-                help=argparse.SUPPRESS,
-            )
 
         return parser
         # return clientmanager.build_plugin_option_parser(parser)
@@ -306,7 +340,7 @@ class OpenStackShell(app.App):
     * super()
     * _final_defaults()
     * OpenStackConfig
-    * get_one_cloud
+    * get_one
     * _load_plugins()
     * _load_commands()
     * ClientManager
@@ -364,7 +398,7 @@ class OpenStackShell(app.App):
         # Parent __init__ parses argv into self.options
         super(OpenStackShell, self).initialize_app(argv)
         self.log.info("START with options: %s",
-                      strutils.mask_password(self.command_options))
+                      strutils.mask_password(" ".join(self.command_options)))
         self.log.debug("options: %s",
                        strutils.mask_password(self.options))
 
@@ -372,10 +406,9 @@ class OpenStackShell(app.App):
         self._final_defaults()
 
         # Do configuration file handling
-        # Ignore the default value of interface. Only if it is set later
-        # will it be used.
         try:
-            cc = cloud_config.OSC_Config(
+            self.cloud_config = cloud_config.OSC_Config(
+                pw_func=prompt_for_password,
                 override_defaults={
                     'interface': None,
                     'auth_type': self._auth_type,
@@ -393,7 +426,7 @@ class OpenStackShell(app.App):
 
         # NOTE(dtroyer): Need to do this with validate=False to defer the
         #                auth plugin handling to ClientManager.setup_auth()
-        self.cloud = cc.get_one_cloud(
+        self.cloud = self.cloud_config.get_one(
             cloud=self.options.cloud,
             argparse=self.options,
             validate=False,
@@ -401,7 +434,7 @@ class OpenStackShell(app.App):
 
         self.log_configurator.configure(self.cloud)
         self.dump_stack_trace = self.log_configurator.dump_trace
-        self.log.debug("defaults: %s", cc.defaults)
+        self.log.debug("defaults: %s", self.cloud_config.defaults)
         self.log.debug("cloud cfg: %s",
                        strutils.mask_password(self.cloud.config))
 
@@ -432,7 +465,7 @@ class OpenStackShell(app.App):
         )
 
         # NOTE(dtroyer): If auth is not required for a command, skip
-        #                get_one_Cloud()'s validation to avoid loading plugins
+        #                get_one()'s validation to avoid loading plugins
         validate = cmd.auth_required
 
         # NOTE(dtroyer): Save the auth required state of the _current_ command
@@ -440,10 +473,13 @@ class OpenStackShell(app.App):
         self.client_manager._auth_required = cmd.auth_required
 
         # Validate auth options
-        self.cloud = self.cloud_config.get_one_cloud(
+        self.cloud = self.cloud_config.get_one(
             cloud=self.options.cloud,
             argparse=self.options,
             validate=validate,
+            app_name=self.client_manager._app_name,
+            app_version=self.client_manager._app_version,
+            additional_user_agent=[('osc-lib', version.version_string)],
         )
         # Push the updated args into ClientManager
         self.client_manager._cli_options = self.cloud
